import socket
import logging
import threading
from .utils import ProxyUtils  # 使用相对导入
from .mqtt_client import MQTTClient  # 使用相对导入
from commands.command_heartbeat import Command0B0CH
from commands.command_02 import Command02
from commands.command_03 import Command03
from commands.command_07 import Command07
from commands.command_08 import Command08
from commands.command_09 import Command09
from commands.command_0A import Command0A
from commands.command_25 import Command25
from commands.command_30 import Command30
from commands.command_19_1A import Command191A
from commands.command_21_22 import Command2122
from commands.command_23_24 import Command2324
from commands.command_26_27 import Command2627




class ChargingPileProxyServer:
    def __init__(self, listen_host='0.0.0.0', listen_port=52461,
                 forward_host='139.9.209.227', forward_port=52461):
        self.listen_host = listen_host
        self.listen_port = listen_port
        self.forward_host = forward_host
        self.forward_port = forward_port
        self.server_socket = None
        self.running = False
        self.clients = {}
        self.remote_connections = {}
        self.mqtt_client = MQTTClient()
        self.pile_ids = {}
        self.utils = ProxyUtils()
        self.command_handler = Command0B0CH()
        self.command_handler = Command02()
        self.command_handler = Command03()
        self.command_handler = Command07()
        self.command_handler = Command08()
        self.command_handler = Command09()
        self.command_handler = Command0A()
        self.command_handler = Command25()
        self.command_handler = Command30()
        self.command_handler = Command191A()
        self.command_handler = Command2122()
        self.command_handler = Command2324()
        self.command_handler = Command2627()

        # 存储登录信息的字典，以桩号为键
        self.login_info = {}
        # 存储对时信息的字典
        self.time_sync_info = {}
        # 存储遥信信息的字典
        self.remote_signal_info = {}
        # 存储故障信息的字典
        self.fault_info = {}
        # 存储心跳信息的字典
        self.heartbeat_info = {}




    def start(self):
        """启动代理服务器"""
        try:
            self.mqtt_client.connect()

            self.server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.server_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            self.server_socket.bind((self.listen_host, self.listen_port))
            self.server_socket.listen(5)
            self.running = True

            start_msg = f"代理服务器已启动，监听地址: {self.listen_host}:{self.listen_port}"
            logging.info(start_msg)
            print(start_msg)
            self.mqtt_client.publish_message(start_msg)

            while self.running:
                client_socket, address = self.server_socket.accept()
                client_msg = f"收到新的客户端连接，地址: {address}"
                logging.info(client_msg)
                print(client_msg)
                self.mqtt_client.publish_message(client_msg)

                client_thread = threading.Thread(target=self.handle_client,
                                              args=(client_socket, address))
                client_thread.daemon = True
                client_thread.start()

        except Exception as e:
            error_msg = f"代理服务器错误: {str(e)}"
            logging.error(error_msg)
            print(error_msg)
            self.mqtt_client.publish_message(error_msg)
        finally:
            if self.server_socket:
                self.server_socket.close()

    def create_remote_connection(self, client_address):
        """创建与远程服务器的连接"""
        try:
            remote_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            remote_socket.connect((self.forward_host, self.forward_port))

            connect_msg = f"已连接到远程服务器 {self.forward_host}:{self.forward_port}"
            logging.info(connect_msg)
            print(connect_msg)
            self.mqtt_client.publish_message(connect_msg)

            self.remote_connections[client_address] = remote_socket
            return remote_socket
        except Exception as e:
            error_msg = f"连接远程服务器失败: {str(e)}"
            logging.error(error_msg)
            print(error_msg)
            self.mqtt_client.publish_message(error_msg)
            return None

    def forward_data(self, source_socket, destination_socket, source_address):
        """转发数据"""
        try:
            while self.running:
                data = source_socket.recv(1024)  # 接收数据
                if not data:
                    break  # 如果没有数据，退出循环

                # 如果接收到的数据长度大于等于14字节，且以'JX'开头
                if len(data) >= 14 and data[0:2] == b'JX':
                    command = data[2]  # 提取命令字节

                    # 根据命令字节处理不同命令
                    if command == 0x01:
                        logging.info(f"处理 01H 命令，数据内容: {data.hex()}")
                        self.command_handler.parse_01h(data)

                    elif command == 0x02:
                        logging.info(f"处理 02H 命令，数据内容: {data.hex()}")
                        self.command_handler.parse_02h(data)

                    elif command == 0x03:
                        logging.info(f"处理 03H 命令，数据内容: {data.hex()}")
                        self.command_handler.parse_03h(data)

                    elif command == 0x07:
                        logging.info(f"处理 07H 命令，数据内容: {data.hex()}")
                        self.command_handler.parse_07h(data)

                    elif command == 0x08:
                        logging.info(f"处理 08H 命令，数据内容: {data.hex()}")
                        self.command_handler.parse_08h(data)

                    elif command == 0x09:
                        logging.info(f"处理 09H 命令，数据内容: {data.hex()}")
                        self.command_handler.parse_09h(data)

                    elif command == 0x0A:
                        logging.info(f"处理 0AH 命令，数据内容: {data.hex()}")
                        self.command_handler.parse_0Ah(data)

                    #其他命令待添加...

                    else:
                        # 未知命令，日志记录
                        logging.warning(f"未知命令：{command:02X}，数据内容: {data.hex()}")

                # 将数据转发到远程或充电桩，判断方向
                if source_socket not in self.remote_connections.values():
                    # 这里是判断是否是客户端连接，进行桩ID提取
                    pile_id = self.utils.extract_pile_id(data)
                    if pile_id:
                        self.pile_ids[source_address] = pile_id

                # 获取本地和远程端口信息
                source_local, source_remote = self.utils.get_socket_info(source_socket)
                dest_local, dest_remote = self.utils.get_socket_info(destination_socket)

                # 判断数据发送方向：是发送到远程服务器还是充电桩
                is_to_remote = destination_socket in self.remote_connections.values()
                direction = "发送到远程服务器" if is_to_remote else "发送到充电桩"
                mqtt_direction = "u" if is_to_remote else "d"

                # 发送数据到目的地
                destination_socket.send(data)

                # 记录数据转发日志
                msg = f"数据转发成功: {direction} | 本地地址: {source_local} | 远程地址: {dest_remote} | 数据长度: {len(data)}"
                logging.info(msg)
                self.mqtt_client.publish_message(msg)

                # 每次数据转发完成后，处理其他相关操作
                # 例如检查是否需要断开连接等
                if not self.running:
                    break

        except Exception as e:
            # 异常处理，日志记录错误信息
            logging.error(f"转发数据出错: {str(e)}")
            print(f"转发数据出错: {str(e)}")
            self.mqtt_client.publish_message(f"转发数据出错: {str(e)}")
            # 若需要，可以选择关闭连接
            if source_socket:
                source_socket.close()
            if destination_socket:
                destination_socket.close()

    def handle_client(self, client_socket, client_address):
        """处理客户端连接"""
        try:
            remote_socket = self.create_remote_connection(client_address)
            if not remote_socket:
                client_socket.close()
                return

            forward_thread = threading.Thread(
                target=self.forward_data,
                args=(client_socket, remote_socket, client_address)
            )
            backward_thread = threading.Thread(
                target=self.forward_data,
                args=(remote_socket, client_socket, client_address)
            )

            forward_thread.daemon = True
            backward_thread.daemon = True

            forward_thread.start()
            backward_thread.start()

            forward_thread.join()
            backward_thread.join()

        except Exception as e:
            error_msg = f"处理客户端连接错误: {str(e)}"
            logging.error(error_msg)
            print(error_msg)
            self.mqtt_client.publish_message(error_msg)
        finally:
            if client_address in self.remote_connections:
                self.remote_connections[client_address].close()
                del self.remote_connections[client_address]
            if client_address in self.pile_ids:
                del self.pile_ids[client_address]
            client_socket.close()
            close_msg = f"客户端连接已关闭 {client_address}"
            logging.info(close_msg)
            print(close_msg)
            self.mqtt_client.publish_message(close_msg)

    def stop(self):
        """停止代理服务器"""
        self.running = False
        if self.server_socket:
            self.server_socket.close()

        for remote_socket in self.remote_connections.values():
            remote_socket.close()
        self.remote_connections.clear()
        self.pile_ids.clear()

        self.mqtt_client.disconnect()

        stop_msg = "代理服务器已停止"
        logging.info(stop_msg)
        print(stop_msg)
        self.mqtt_client.publish_message(stop_msg)